@@ -54,7 +54,7 @@
 #include "totem-glow-button.h"
 #include "video-utils.h"
 
-#include "totem-plugin-viewer-commands.h"
+#include "totem-plugin-viewer-constants.h"
 #include "totem-plugin-viewer-options.h"
 #include "totempluginviewer-marshal.h"
 
@@ -70,19 +70,13 @@ GtkWidget *totem_pp_create (void);
 #define DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT 0
 #endif
 
-typedef enum {
-	STATE_PLAYING,
-	STATE_PAUSED,
-	STATE_STOPPED,
-	LAST_STATE
-} TotemStates;
-
 typedef enum {
 	TOTEM_PLUGIN_TYPE_BASIC,
 	TOTEM_PLUGIN_TYPE_GMP,
 	TOTEM_PLUGIN_TYPE_COMPLEX,
 	TOTEM_PLUGIN_TYPE_NARROWSPACE,
 	TOTEM_PLUGIN_TYPE_MULLY,
+	TOTEM_PLUGIN_TYPE_CONE,
 	TOTEM_PLUGIN_TYPE_LAST
 } TotemPluginType;
 
@@ -194,8 +188,10 @@ enum {
 	BUTTON_PRESS,
 	START_STREAM,
 	STOP_STREAM,
+	SIGNAL_TICK,
 	LAST_SIGNAL
 };
+
 static int signals[LAST_SIGNAL] = { 0 };
 
 static void
@@ -220,12 +216,13 @@ totem_embedded_finalize (GObject *object)
 
 static void totem_embedded_class_init (TotemEmbeddedClass *klass)
 {
-	GType param_types[2];
+	GType param_types[3];
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = totem_embedded_finalize;
 
 	param_types[0] = param_types[1] = G_TYPE_UINT;
+	param_types[2] = G_TYPE_STRING;
 	signals[BUTTON_PRESS] =
 		g_signal_newv ("button-press",
 				G_TYPE_FROM_CLASS (object_class),
@@ -252,6 +249,14 @@ static void totem_embedded_class_init (TotemEmbeddedClass *klass)
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE,
 				0, NULL);
+	signals[SIGNAL_TICK] =
+		g_signal_newv ("tick",
+				G_TYPE_FROM_CLASS (object_class),
+				G_SIGNAL_RUN_LAST,
+				NULL /* class closure */,
+				NULL /* accu */, NULL /* accu data */,
+				totempluginviewer_marshal_VOID__UINT_UINT_STRING,
+				G_TYPE_NONE, 3, param_types);
 }
 
 static void
@@ -294,22 +299,16 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 {
 	GtkWidget *image;
 	gchar *id;
-	static const char *states[] = {
-		"PLAYING",
-		"PAUSED",
-		"STOPPED",
-		"INVALID"
-	};
 
 	if (state == emb->state)
 		return;
 
-	g_message ("Viewer state: %s", states[state]);
+	g_message ("Viewer state: %s", totem_states[state]);
 
 	image = gtk_button_get_image (GTK_BUTTON (emb->pp_button));
 
 	switch (state) {
-	case STATE_STOPPED:
+	case TOTEM_STATE_STOPPED:
 		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Stopped"));
 		totem_statusbar_set_time_and_length (emb->statusbar, 0, 0);
@@ -321,11 +320,11 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 				 emb->cursor);
 		}
 		break;
-	case STATE_PAUSED:
+	case TOTEM_STATE_PAUSED:
 		id = GTK_STOCK_MEDIA_PLAY;
 		totem_statusbar_set_text (emb->statusbar, _("Paused"));
 		break;
-	case STATE_PLAYING:
+	case TOTEM_STATE_PLAYING:
 		id = GTK_STOCK_MEDIA_PAUSE;
 		totem_statusbar_set_text (emb->statusbar, _("Playing"));
 		if (emb->href_uri == NULL && emb->hidden == FALSE) {
@@ -340,7 +339,7 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 	}
 
 	if (emb->scrsaver != NULL)
-		totem_scrsaver_set_state (emb->scrsaver, (state == STATE_PLAYING) ? FALSE : TRUE);
+		totem_scrsaver_set_state (emb->scrsaver, (state == TOTEM_STATE_PLAYING) ? FALSE : TRUE);
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
 	gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (emb->pp_fs_button), id);
 
@@ -387,7 +386,7 @@ totem_embedded_set_logo_by_name (TotemEmbedded *embedded,
 	GdkPixbuf *logo, *padded;
 	int size, width, height;
 
-	totem_embedded_set_state (embedded, STATE_STOPPED);
+	totem_embedded_set_state (embedded, TOTEM_STATE_STOPPED);
 
 	if (embedded->audioonly != FALSE)
 		return;
@@ -460,7 +459,7 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		/* FIXME we haven't even started sending yet! */
 		//g_signal_emit (emb, signals[STOP_STREAM], 0);
 
-		totem_embedded_set_state (emb, STATE_STOPPED);
+		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 		totem_embedded_set_logo_by_name (emb, "image-missing");
 
 		errint = g_error_new (TOTEM_EMBEDDED_ERROR_QUARK,
@@ -499,10 +498,12 @@ totem_embedded_play (TotemEmbedded *emb,
 {
 	GError *err = NULL;
 
+	g_return_val_if_fail (emb->current_uri != NULL, FALSE);
+
 	totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
 
 	if (bacon_video_widget_play (emb->bvw, &err) != FALSE) {
-		totem_embedded_set_state (emb, STATE_PLAYING);
+		totem_embedded_set_state (emb, TOTEM_STATE_PLAYING);
 		totem_embedded_set_pp_state (emb, TRUE);
 	} else {
 		g_warning ("Error in bacon_video_widget_play: %s", err->message);
@@ -516,8 +517,8 @@ static gboolean
 totem_embedded_pause (TotemEmbedded *emb,
 		      GError **error)
 {
+	totem_embedded_set_state (emb, TOTEM_STATE_PAUSED);
 	bacon_video_widget_pause (emb->bvw);
-	totem_embedded_set_state (emb, STATE_PAUSED);
 
 	return TRUE;
 }
@@ -526,8 +527,8 @@ static gboolean
 totem_embedded_stop (TotemEmbedded *emb,
 		     GError **error)
 {
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 	bacon_video_widget_stop (emb->bvw);
-	totem_embedded_set_state (emb, STATE_STOPPED);
 
 	return TRUE;
 }
@@ -592,6 +593,16 @@ totem_embedded_set_error_logo (TotemEmbedded *embedded,
 	return TRUE;
 }
 
+static gboolean
+totem_embedded_set_volume (TotemEmbedded *embedded,
+			   gdouble volume,
+			   GError *error)
+{
+	g_message ("totem_embedded_set_volume: %f", volume);
+	bacon_video_widget_set_volume (embedded->bvw, volume);
+	return TRUE;
+}
+
 /* Copied from nautilus-program-choosing.c */
 
 extern char **environ;
@@ -1194,7 +1205,7 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	emb->about = g_object_new (GTK_TYPE_ABOUT_DIALOG,
 				   "name", _("Totem Browser Plugin"),
 				   "version", VERSION,
-				   "copyright", _("Copyright \xc2\xa9 2002-2006 Bastien Nocera"),
+				   "copyright", _("Copyright \xc2\xa9 2002-2007 Bastien Nocera"),
 				   "comments", description,
 				   "authors", authors,
 				   "translator-credits", _("translator-credits"),
@@ -1254,19 +1265,96 @@ on_preferences1_activate (GtkButton *button, TotemEmbedded *emb)
 static void
 on_play_pause (GtkWidget *widget, TotemEmbedded *emb)
 {
-	if (emb->state == STATE_PLAYING) {
+	if (emb->state == TOTEM_STATE_PLAYING) {
 		totem_embedded_pause (emb, NULL);
 	} else {
 		if (emb->current_uri == NULL) {
 			totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), FALSE);
 			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
-				       GDK_CURRENT_TIME, 0);
+				       gtk_get_current_event_time (), 0);
 		} else {
 			totem_embedded_play (emb, NULL);
 		}
 	}
 }
 
+static void
+popup_menu_position_func (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *button) 
+{
+	GtkWidget *widget = GTK_WIDGET (button);
+	GtkRequisition menu_req;
+	GtkTextDirection direction;
+
+	gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
+
+	direction = gtk_widget_get_direction (widget);
+
+	gdk_window_get_origin (widget->window, x, y);
+	*x += widget->allocation.x;
+	*y += widget->allocation.y;
+
+	if (direction == GTK_TEXT_DIR_LTR)
+		*x += MAX (widget->allocation.width - menu_req.width, 0);
+	else if (menu_req.width > widget->allocation.width)
+		*x -= menu_req.width - widget->allocation.width;
+
+	/* This might not work properly if the popup button is right at the
+	 * top of the screen, but really, what are the chances */
+	*y -= menu_req.height;
+
+	*push_in = FALSE;
+}
+
+static void
+popup_menu_over_arrow (GtkToggleButton *button,
+		       GtkMenu *menu,
+		       GdkEventButton *event)
+{
+	gtk_menu_popup (menu, NULL, NULL,
+			(GtkMenuPositionFunc) popup_menu_position_func,
+			button,
+			event ? event->button : 0,
+			event ? event->time : gtk_get_current_event_time ());
+}
+
+static void
+on_popup_button_toggled (GtkToggleButton *button, TotemEmbedded *emb)
+{
+	GtkMenu *menu;
+
+	menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+
+	if (gtk_toggle_button_get_active (button) && !GTK_WIDGET_VISIBLE (menu)) {
+		/* we get here only when the menu is activated by a key
+		 * press, so that we can select the first menu item */
+		popup_menu_over_arrow (button, menu, NULL);
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+	}
+}
+
+static gboolean
+on_popup_button_button_pressed (GtkToggleButton *button,
+				GdkEventButton *event,
+				TotemEmbedded *emb)
+{
+	if (event->button == 1) {
+		GtkMenu *menu;
+
+		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+		if (!GTK_WIDGET_VISIBLE (menu)) {
+			popup_menu_over_arrow (button, menu, event);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+		} else {
+			gtk_menu_popdown (menu);
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
+		}
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
 on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 {
@@ -1279,13 +1367,13 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	/* We are using a local cache, so we resolve against the stream_uri */
 	if (emb->stream_uri)
-		new_uri = totem_resolve_relative_link (emb->stream_uri, mrl);
+		new_uri = totem_pl_parser_resolve_url (emb->stream_uri, mrl);
 	/* We don't have a local cache, so resolve against the URI */
 	else if (emb->current_uri)
-		new_uri = totem_resolve_relative_link (emb->current_uri, mrl);
+		new_uri = totem_pl_parser_resolve_url (emb->current_uri, mrl);
 	/* FIXME: not sure that this is actually correct... */
 	else if (emb->base_uri)
-		new_uri = totem_resolve_relative_link (emb->base_uri, mrl);
+		new_uri = totem_pl_parser_resolve_url (emb->base_uri, mrl);
 
 	if (!new_uri)
 		return;
@@ -1297,7 +1385,7 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 
 	totem_embedded_set_uri (emb, new_uri, emb->base_uri /* FIXME? */, FALSE);
 
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
 	totem_embedded_open_internal (emb, TRUE, NULL /* FIXME? */);
 }
@@ -1354,32 +1442,33 @@ on_video_button_press_event (BaconVideoWidget *bvw,
 			     TotemEmbedded *emb)
 {
 	guint state = event->state & gtk_accelerator_get_default_mod_mask ();
-	gboolean handled = FALSE;
+	GtkMenu *menu;
 
 	//g_print ("button-press type %d button %d state %d play-state %d\n",
 	//	 event->type, event->button, state, emb->state);
 
-	if (event->type == GDK_BUTTON_PRESS &&
-	    event->button == 1 &&
-	    state == 0 &&
-	    emb->state == STATE_STOPPED) {
-		g_message ("emitting signal");
-		g_signal_emit (emb, signals[BUTTON_PRESS], 0,
-			       event->time,
-			       event->button);
-	} else if (event->type == GDK_BUTTON_PRESS &&
-		   event->button == 3 &&
-		   state == 0) {
-		GtkMenu *menu;
+	menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
+
+	if (event->type == GDK_BUTTON_PRESS && event->button == 1 && state == 0 && emb->state == TOTEM_STATE_STOPPED) {
+		if (!GTK_WIDGET_VISIBLE (menu)) {
+			g_message ("emitting signal");
+			g_signal_emit (emb, signals[BUTTON_PRESS], 0,
+				       event->time,
+				       event->button);
+		} else {
+			gtk_menu_popdown (menu);
+		}
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 3 && state == 0) {
+		GtkWidget *popup_button;
+
+		popup_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (popup_button), FALSE);
 
-		menu = GTK_MENU (gtk_builder_get_object (emb->menuxml, "menu"));
 		gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
 				event->button, event->time);
-
-		handled = TRUE;
 	}
 
-	return handled;
+	return FALSE;
 }
 
 static void
@@ -1387,7 +1476,7 @@ on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 {
 	gboolean start_play;
 
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 	gtk_adjustment_set_value (emb->seekadj, 0);
 
 	/* FIXME: the plugin needs to handle EOS itself, e.g. for QTNext */
@@ -1444,7 +1533,7 @@ on_error_event (BaconVideoWidget *bvw,
 		if (emb->is_browser_stream)
 			g_signal_emit (emb, signals[STOP_STREAM], 0);
 	
-		totem_embedded_set_state (emb, STATE_STOPPED);
+		totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
 		/* If we have a playlist, and that the current item
 		 * is < 60 seconds long, just go through it
@@ -1483,11 +1572,11 @@ static void
 on_tick (GtkWidget *bvw,
 		gint64 current_time,
 		gint64 stream_length,
-		float current_position,
+		double current_position,
 		gboolean seekable,
 		TotemEmbedded *emb)
 {
-	if (emb->state != STATE_STOPPED) {
+	if (emb->state != TOTEM_STATE_STOPPED) {
 		gtk_widget_set_sensitive (emb->seek, seekable);
 		gtk_widget_set_sensitive (emb->fs->seek, seekable);
 		if (emb->seeking == FALSE)
@@ -1506,6 +1595,9 @@ on_tick (GtkWidget *bvw,
 			(TOTEM_TIME_LABEL (emb->fs->time_label),
 			 current_time, stream_length);
 	}
+
+	g_signal_emit (emb, signals[SIGNAL_TICK], 0,
+		       (guint32) current_time, (guint32) stream_length, totem_states[emb->state]);
 }
 
 static void
@@ -1515,7 +1607,8 @@ on_buffering (BaconVideoWidget *bvw, guint percentage, TotemEmbedded *emb)
 }
 
 static void
-property_notify_cb_volume (BaconVideoWidget *bvw, GParamSpec *spec,
+property_notify_cb_volume (BaconVideoWidget *bvw,
+			   GParamSpec *spec,
 			   TotemEmbedded *emb)
 {
 	double volume;
@@ -1637,6 +1730,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 			  int height)
 {
 	GtkWidget *child, *container, *image;
+	GtkWidget *popup_button;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
@@ -1763,6 +1857,12 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_signal_connect (G_OBJECT (emb->pp_button), "clicked",
 			  G_CALLBACK (on_play_pause), emb);
 
+	popup_button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
+	g_signal_connect (G_OBJECT (popup_button), "toggled",
+			  G_CALLBACK (on_popup_button_toggled), emb);
+	g_signal_connect (G_OBJECT (popup_button), "button-press-event",
+			  G_CALLBACK (on_popup_button_button_pressed), emb);
+
 	gc = gconf_client_get_default ();
 	volume = ((double) gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL)) / 100.0;
 	g_object_unref (G_OBJECT (gc));
@@ -1816,7 +1916,13 @@ totem_embedded_construct (TotemEmbedded *emb,
 		g_object_unref (rcstyle);
 	}
 
-	totem_embedded_set_state (emb, STATE_STOPPED);
+	/* Create the cursor before setting the state */
+	if (!emb->hidden) {
+		emb->cursor = gdk_cursor_new_for_display
+			(gtk_widget_get_display (emb->window),
+			 GDK_HAND2);
+	}
+	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
 	if (!emb->hidden) {
 		gtk_widget_show (emb->window);
@@ -1827,17 +1933,17 @@ totem_embedded_construct (TotemEmbedded *emb,
 					     GTK_WINDOW (emb->window), emb);
 	g_assert (emb->menuxml);
 
-	/* Create cursor and set the logo */
-	if (!emb->hidden) {
-		emb->cursor = gdk_cursor_new_for_display
-			(gtk_widget_get_display (emb->window),
-			 GDK_HAND2);
+	/* Set the logo and the button glow */
+	if (!emb->hidden && emb->autostart == FALSE) {
+		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
+		if (gtk_widget_get_direction (GTK_WIDGET (emb->bvw)) == GTK_TEXT_DIR_LTR)
+			totem_embedded_set_logo_by_name (emb, "gtk-media-play-ltr");
+		else
+			totem_embedded_set_logo_by_name (emb, "gtk-media-play-rtl");
+	} else {
 		totem_embedded_set_logo_by_name (emb, "totem");
 	}
 
-	if (!emb->hidden && emb->autostart == FALSE)
-		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
-
 	return TRUE;
 }
 
@@ -1950,39 +2056,6 @@ totem_embedded_clear_playlist (TotemEmbedded *embedded)
 	embedded->num_items = 0;
 }
 
-/* FIXME this should live in the playlist parser */
-static int
-totem_embedded_parse_duration (const char *duration)
-{
-	int hours, minutes, seconds, fractions;
-
-	if (!duration)
-		return -1;
-
-	/* Formats used by both ASX and RAM files */
-	if (sscanf (duration, "%d:%d:%d.%d", &hours, &minutes, &seconds, &fractions) == 4) {
-		int ret = hours * 3600 + minutes * 60 + seconds;
-		if (ret == 0 && fractions > 0)
-			ret = 1;
-		return ret;
-	}
-	if (sscanf (duration, "%d:%d:%d", &hours, &minutes, &seconds) == 3)
-		return hours * 3600 + minutes * 60 + seconds;
-	if (sscanf (duration, "%d:%d.%d", &minutes, &seconds, &fractions) == 3) {
-		int ret = minutes * 60 + seconds;
-		if (ret == 0 && fractions > 0)
-			ret = 1;
-		return ret;
-	}
-	if (sscanf (duration, "%d:%d", &minutes, &seconds) == 2)
-		return minutes * 60 + seconds;
-	/* PLS files format */
-	if (sscanf (duration, "%d", &seconds) == 1)
-		return seconds;
-
-	return -1;
-}
-
 static void
 entry_metadata_foreach (const char *key,
 			const char *value,
@@ -2001,7 +2074,6 @@ entry_parsed (TotemPlParser *parser,
 {
 	TotemEmbedded *emb = (TotemEmbedded *) data;
 	TotemPlItem *item;
-	int duration;
 
 	if (g_str_has_prefix (uri, "file://") != FALSE
 	    && g_str_has_prefix (emb->base_uri, "file://") == FALSE) {
@@ -2012,15 +2084,10 @@ entry_parsed (TotemPlParser *parser,
 	g_print ("added URI '%s'\n", uri);
 	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 
-	/* Skip short advert streams */
-	duration = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "duration"));
-	if (duration == 0)
-		return;
-
 	item = g_new0 (TotemPlItem, 1);
 	item->uri = g_strdup (uri);
-	item->duration = duration;
-	item->starttime = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "starttime"));
+	item->duration = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "duration"), FALSE);
+	item->starttime = totem_pl_parser_parse_duration (g_hash_table_lookup (metadata, "starttime"), FALSE);
 
 	emb->playlist = g_list_prepend (emb->playlist, item);
 }
@@ -2108,7 +2175,8 @@ parse_plugin_type (const gchar *option_name,
 		"gmp",
 		"complex",
 		"narrowspace",
-		"mully"
+		"mully",
+		"cone"
 	};
 	TotemPluginType type;
 
@@ -2267,7 +2335,7 @@ int main (int argc, char **argv)
 
 	emb = g_object_new (TOTEM_TYPE_EMBEDDED, NULL);
 
-	emb->state = LAST_STATE;
+	emb->state = TOTEM_STATE_INVALID;
 	emb->width = -1;
 	emb->height = -1;
 	emb->controller_hidden = arg_no_controls;
